@@ -1,71 +1,26 @@
-# coding: utf-8
-u"""Платформа разработки приложений ERP типа на python и django."""
-from __future__ import absolute_import
+#coding:utf-8
+"""
+Платформа разработки приложений ERP типа на python и django
+"""
 
-from functools import reduce
-from json.encoder import encode_basestring
-from json.encoder import encode_basestring_ascii
 import copy
 import datetime
-import decimal
 import json
+import decimal
 import sys
 
+from django.db import models as dj_models
+from django.http import HttpResponseRedirect, HttpResponseServerError
 from django.conf import settings
 from django.contrib import auth
-from django.db import models as dj_models
-from django.http import HttpResponseRedirect
-from django.http import HttpResponseServerError
 from django.utils import datetime_safe
 from django.views.debug import ExceptionReporter
-from m3_django_compat import ModelOptions
-from m3_django_compat import is_authenticated
-from m3_django_compat.middleware import MiddlewareMixin
 
-from .actions import ApplicationLogicException
-from .actions import OperationResult
-from .actions.urls import get_app_urlpatterns
+import actions
 
+from actions import ApplicationLogicException
 
-def escape_html(s):
-    u"""Экранирует символы HTML разметки.
-
-    :param  basestring s: строка для экранирования
-
-    :rtype: basestring
-    """
-    replace_map = (
-        ('&', '&amp;'),
-        ('<', '&lt;'),
-        ('>', '&gt;'),
-    )
-    return reduce(lambda s, r: s.replace(*r), replace_map, s)
-
-
-def _encode_basestring(s):
-    """Возвращает представление Python строк для JSON.
-
-    Расширяет функцию :func:`json.encoder.encode_basestring` поддержкой
-    строк, помеченных с помощью :func:`~django.utils.safestring.mark_safe`.
-    """
-    for_escape = not hasattr(s, '__html__')
-    result = encode_basestring(s)
-    if for_escape:
-        result = escape_html(result)
-    return result
-
-
-def _encode_basestring_ascii(s):
-    """Возвращает только ASCII представление Python строк для JSON .
-
-    Расширяет функцию :func:`json.encoder.encode_basestring_ascii` поддержкой
-    строк, помеченных с помощью :func:`~django.utils.safestring.mark_safe`.
-    """
-    for_escape = not hasattr(s, '__html__')
-    result = encode_basestring_ascii(s)
-    if for_escape:
-        result = escape_html(result)
-    return result
+from actions.urls import get_app_urlpatterns
 
 
 def date2str(date, template=None):
@@ -76,10 +31,8 @@ def date2str(date, template=None):
     # This library does not support strftime's \"%s\" or \"%y\" format strings.
     # Allowed if there's an even number of \"%\"s because they are escaped.
     """
-    default_format = getattr(settings, 'PYTHON_DATE_FORMAT', '%d.%m.%Y')
     return datetime_safe.new_datetime(date).strftime(
-        template or default_format
-    )
+        template or settings.DATE_FORMAT or '%d.%m.%Y')
 
 
 class AutoLogout(object):
@@ -99,11 +52,11 @@ class AutoLogout(object):
 
         # У аутентифицированного пользователя проверяем таймаут,
         # а ананимусов сразу посылаем
-        if is_authenticated(request.user):
+        if request.user.is_authenticated():
             last_time = request.session.get(self.session_key, None)
             if last_time is not None:
                 delta = datetime.datetime.now() - last_time
-                if delta.seconds // 60 > settings.INACTIVE_SESSION_LIFETIME:
+                if delta.seconds / 60 > settings.INACTIVE_SESSION_LIFETIME:
                     # После логаута сессия уже другая
                     # и присваивать время не нужно
                     auth.logout(request)
@@ -113,39 +66,11 @@ class AutoLogout(object):
             request.session[self.session_key] = datetime.datetime.now()
 
 
-class _EncodeFunctionsPatcher(object):
-    u"""Менеджер контекста для подмены кодировщиков в ``M3JSONEncoder``
-
-    Поскольку функций кодировщиков нет в самом инстансе класса
-    ``M3JSONEncoder`` чтобы не переписывать родительский метод с заменой 2
-    переменных заменяем их с помощью менеджера контекста через глобальные
-    переменные метода iterencode на содержащие возможность экранирования HTML
-    функции, после использования возвращаем обратно.
-    """
-
-    def __init__(self, encoder):
-        self.encoder = encoder
-        self._globals = self.encoder.iterencode.__func__.__globals__
-
-    def __enter__(self):
-        self._globals['encode_basestring'] = _encode_basestring
-        self._globals['encode_basestring_ascii'] = _encode_basestring_ascii
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self._globals['encode_basestring'] = encode_basestring
-        self._globals['encode_basestring_ascii'] = encode_basestring_ascii
-
-
 class M3JSONEncoder(json.JSONEncoder):
     def __init__(self, *args, **kwargs):
         self.dict_list = kwargs.pop('dict_list', None)
         super(M3JSONEncoder, self).__init__(*args, **kwargs)
 
-    def iterencode(self, *args, **kwargs):
-        # передаем через super, т.к. нам нужны переменные родительского класса
-        with _EncodeFunctionsPatcher(super(M3JSONEncoder, self)):
-            return super(M3JSONEncoder, self).iterencode(*args, **kwargs)
-
     def default(self, obj):
         # обработаем простейшие объекты,
         # которые не обрабатываются стандартным способом
@@ -159,6 +84,13 @@ class M3JSONEncoder(json.JSONEncoder):
         elif isinstance(obj, decimal.Decimal):
             return str(obj)
 
+        # тут элементы, имеющие атрибут _config (Ext-контролы, обычно)
+        # возвращаются в виде собственно конфига
+        try:
+            return obj._config
+        except AttributeError:
+            pass
+
         # Прошерстим методы и свойства, найдем те,
         # которые могут передаваться на клиента
         # Клонирование словаря происходит потому,
@@ -175,8 +107,8 @@ class M3JSONEncoder(json.JSONEncoder):
         related_objs_attrs = []
         manager_names = []
         if isinstance(obj, dj_models.Model):
-            related_objs = ModelOptions(obj).get_all_related_objects()
-            related_objs_attrs = [ro.model_name for ro in related_objs]
+            related_objs = obj._meta.get_all_related_objects()
+            related_objs_attrs = [ro.var_name for ro in related_objs]
             # Также соберем все атрибуты-менеджеры (их может быть несколько).
             # Сюда попадет "objects", который исключаем из обработки ниже.
             for attr in obj.__class__.__dict__:
@@ -243,7 +175,7 @@ class M3JSONEncoder(json.JSONEncoder):
                     # если метод или свойство есть в классе,
                     # то проверим у него признак
                     class_attr_value = getattr(obj.__class__, attr, None)
-                    if class_attr_value is not None:
+                    if not class_attr_value is None:
                         json_encode = getattr(
                             class_attr_value, 'json_encode', False)
                         if json_encode:
@@ -254,7 +186,7 @@ class M3JSONEncoder(json.JSONEncoder):
                             else:
                                 # иначе это было свойство или какой-то атрибут
                                 dict[attr] = value
-                except Exception as exc:
+                except Exception, exc:
                     # Вторая проблема с моделями в том,
                     # что dir кроме фактических полей возвращает ассессоры.
                     # При попытке обратиться к ним происходит запрос(!)
@@ -280,7 +212,7 @@ class M3JSONEncoder(json.JSONEncoder):
                         else:
                             cleaned_dict[field_name + '_ref_name'] = getattr(
                                 getattr(obj, field_name), 'name')
-                except:  # noqa
+                except:
                     pass
             if len(attribute) > 6 and attribute.endswith('_cache'):
                 # вережим этот кусок, т.к. если есть кэш на ForeignKey,
@@ -328,9 +260,9 @@ def authenticated_user_required(f):
 
     def action(request, *args, **kwargs):
         user = request.user
-        if not user or not is_authenticated(user):
+        if not user or not user.is_authenticated():
             if request.is_ajax():
-                res = OperationResult.by_message(
+                res = actions.OperationResult.by_message(
                     u'Вы не авторизованы. Возможно, закончилось время '
                     u'пользовательской сессии.<br>'
                     u'Для повторной аутентификации обновите страницу.')
@@ -343,11 +275,11 @@ def authenticated_user_required(f):
     return action
 
 
-class PrettyTracebackMiddleware(MiddlewareMixin):
+class PrettyTracebackMiddleware(object):
     """
     Middleware, выводящая traceback'и в html-виде
     """
     def process_exception(self, request, exception):
         reporter = ExceptionReporter(request, *sys.exc_info())
         html = reporter.get_traceback_html()
-        return HttpResponseServerError(html, content_type='text/html')
+        return HttpResponseServerError(html, mimetype='text/html')
