@@ -1,56 +1,59 @@
-# coding:utf-8
+#coding:utf-8
 """
-Основные объекты библиотеки: механизмы проверки прав, экшены, паки,
-контроллеры, кэш контроллеров
+Основные объекты библиотеки: механизмы проверки прав, экшены, паки, контроллеры, кэш контроллеров
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 """
-from __future__ import absolute_import
-from __future__ import print_function
-
-from functools import wraps
-from logging import getLogger
 import abc
-import importlib
+from functools import wraps
 import inspect
-import re
+import importlib
 import threading
+import re
 import warnings
 
 from django import http
 from django.conf import settings
-from m3_django_compat import get_installed_apps
-from six import iteritems
-from six import string_types
-from six import text_type
-from six import with_metaclass
-from six.moves import map
-import django
+from django.utils.importlib import import_module
 
-from .context import ActionContext
-from .context import ActionContextDeclaration
-from .context import ContextBuildingError
-from .context import CriticalContextBuildingError
-from .context import DeclarativeActionContext
-from .context import RequiredFailed
-from .exceptions import ActionException
-from .exceptions import ActionNotFoundException
-from .exceptions import ActionPackNotFoundException
-from .exceptions import ActionUrlIsNotDefined
-from .exceptions import ApplicationLogicException
-from .exceptions import ReinitException
-from .metrics import create_statsd_client
-from .results import ActionRedirectResult
-from .results import ActionResult
-from .results import BaseContextedResult
-from .results import HttpReadyResult
-from .results import JsonResult
-from .results import OperationResult
-from .results import PreJsonResult
-from .results import TextResult
-from .results import XMLResult
+try:
+    from django.utils.log import logger
+except ImportError:
+    from django.utils.log import getLogger
+    logger = getLogger('django')
 
+from results import (
+    ActionResult,
+    PreJsonResult,
+    JsonResult,
+    HttpReadyResult,
+    TextResult,
+    XMLResult,
+    BaseContextedResult,
+    OperationResult,
+    ActionRedirectResult
+)
 
-logger = getLogger('django')
+from exceptions import (
+    ApplicationLogicException,
+    ActionException,
+    ActionNotFoundException,
+    ActionPackNotFoundException,
+    ReinitException,
+    ActionUrlIsNotDefined
+)
 
+import utils
+
+from context import (
+    ActionContext,
+    ActionContextDeclaration,
+    DeclarativeActionContext,
+    RequiredFailed,
+    ContextBuildingError,
+    CriticalContextBuildingError,
+)
+
+from metrics import create_statsd_client
 
 _STATSD_CLIENT = create_statsd_client(settings)
 
@@ -58,8 +61,7 @@ _STATSD_CLIENT = create_statsd_client(settings)
 ACD = ActionContextDeclaration
 
 
-def _clean_url(s):
-    return re.sub(r'^[/^]*(.*?)[$/]*$', r'\1', s)
+_clean_url = lambda s: re.sub(r'^[/^]*(.*?)[$/]*$', r'\1', s)
 
 
 def _import_by_path(path):
@@ -131,7 +133,7 @@ def _must_be_replaced_by(alternative):
             warnings.warn(
                 "'{clazz}.{method}' deprecated! Use '{alternative}'!".format(
                     clazz=class_name,
-                    method=fn.__name__,
+                    method=fn.func_name,
                     alternative=alternative
                 ), FutureWarning, 2)
             return fn(self, *args, **kwargs)
@@ -139,16 +141,17 @@ def _must_be_replaced_by(alternative):
     return wrapper
 
 
-# =============================================================================
+#==============================================================================
 # Абстрактный механизм проверки прав и его реализация,
 # проверяющая права через механизм django.contrib.auth.models.User.has_perm
-# =============================================================================
+#==============================================================================
 
 
-class AbstractPermissionChecker(with_metaclass(abc.ABCMeta, object)):
+class AbstractPermissionChecker(object):
     """
     Абстрактный механизм проверки прав
     """
+    __metaclass__ = abc.ABCMeta
 
     @abc.abstractmethod
     def has_action_permission(self, request, action, subpermission=None):
@@ -377,7 +380,7 @@ def _permission_checker_fabric():
         if path is None:
             # если backend не задан в настройках, то исходим из того,
             # подключены ли пользователи Django
-            if 'django.contrib.auth' in get_installed_apps():
+            if 'django.contrib.auth' in settings.INSTALLED_APPS:
                 result = AuthUserPermissionChecker
             else:
                 result = BypassPermissionChecker
@@ -400,9 +403,9 @@ def _permission_checker_fabric():
 _permission_checker = LazyContainer(_permission_checker_fabric)
 
 
-# ========================== ИСКЛЮЧЕНИЯ =======================================
+#========================== ИСКЛЮЧЕНИЯ ========================================
 
-# =============================================================================
+#==============================================================================
 class Action(object):
     u"""
     Базовый класс, от которого должны наследоваться все Action'ы в системе.
@@ -464,7 +467,7 @@ class Action(object):
         # метод оставлен для совместимости
         return self.has_perm(request)
 
-    # ================= НОВЫЙ ИНТЕРФЕЙС ПРОВЕРКИ ПРАВ =========================
+    #================= НОВЫЙ ИНТЕРФЕЙС ПРОВЕРКИ ПРАВ ==========================
     def has_perm(self, request, subpermission=None):
         """
         Интерфейсный метод проверки (под)прав экшна.
@@ -483,7 +486,7 @@ class Action(object):
         """
         # по-умолчанию код генерится backend`ом
         return _permission_checker.get_perm_code(self, subpermission)
-    # =========================================================================
+    #==========================================================================
 
     def pre_run(self, request, context):
         """
@@ -521,6 +524,24 @@ class Action(object):
         """
         pass
 
+    def build_context(self, request, suffix=None):
+        """
+        Возвращает заполненый экземпляр ActionContext
+        :param request: запрос
+        :type request: django.http.Request
+
+        :return: экземпляр ActionContext (или потомка)
+        :rtype: m3_core.actions.context.ActionContext
+        """
+        rules = self.context_declaration()
+        if DeclarativeActionContext.matches(rules):
+            context = DeclarativeActionContext()
+        else:
+            context = ActionContext()
+        if rules is not None:
+            context.build(request, rules)
+        return context
+
     def context_declaration(self):
         """
         Метод объявления необходимости наличия
@@ -591,7 +612,7 @@ class Action(object):
         Возвращает полный путь от хоста до конечного экшена
         @deprecated: Дублирует absolute_url
         '''
-        # TODO: Переписать, т.к. этот код дублирует функции контроллера
+        #TODO: Переписать, т.к. этот код дублирует функции контроллера
         assert isinstance(self.controller, ActionController), (
             '%s is not actioncontroller in %s' % (self.controller, self))
         # Очищаем от мусора рег. выр.
@@ -705,7 +726,7 @@ class ActionPack(object):
                 pack = pack.parent
         else:
             path = [pack.url]
-        return '/'.join([pack.controller.url] + list(map(_clean_url, path)))
+        return '/'.join([pack.controller.url] + map(_clean_url, path))
 
     def get_permission_code(self):
         # метод оставлен для совместимости
@@ -720,7 +741,7 @@ class ActionPack(object):
 
         return self.has_perm(request, sub_code)
 
-    # ================= НОВЫЙ ИНТЕРФЕЙС ПРОВЕРКИ ПРАВ =========================
+    #================= НОВЫЙ ИНТЕРФЕЙС ПРОВЕРКИ ПРАВ ==========================
     def has_perm(self, request, permission):
         """
         Интерфейсный метод проверки подправ пака.
@@ -739,7 +760,7 @@ class ActionPack(object):
         """
         # по-умолчанию код генерится backend`ом
         return _permission_checker.get_perm_code(self, permission)
-    # =========================================================================
+    #==========================================================================
 
     def pre_run(self, request, context):
         """
@@ -857,9 +878,9 @@ class ActionController(object):
             else super(ActionController, self).__str__()
         )
 
-    # =========================================================================
+    #==========================================================================
     # Методы для быстрого поиска экшенов и паков по разным атрибутам
-    # =========================================================================
+    #==========================================================================
     def _add_action_to_search_dicts(self, action, full_path):
         """ Добавляет экшен в словари для быстрого доступа """
         assert isinstance(action, Action)
@@ -881,14 +902,14 @@ class ActionController(object):
         """
         self._actions_by_name.clear()
         self._actions_by_type.clear()
-        for full_path, v in iteritems(self._url_patterns):
+        for full_path, v in self._url_patterns.iteritems():
             _, action = v
             self._add_action_to_search_dicts(action, full_path)
         # Обновлять _packs_by_name и _packs_by_type не нужно!
 
-    # =========================================================================
+    #==========================================================================
     # Методы формирующие иерархию экшенов и паков
-    # =========================================================================
+    #==========================================================================
     def _load_class(self, full_path):
         '''
         По полному пути загружает и созвращает класс
@@ -898,7 +919,7 @@ class ActionController(object):
         mod_name = full_path[:dot]
         pack_name = full_path[dot + 1:]
         # Пробуем загрузить
-        mod = importlib.import_module(mod_name)
+        mod = import_module(mod_name)
         clazz = getattr(mod, pack_name)
         return clazz
 
@@ -929,8 +950,8 @@ class ActionController(object):
                 self._build_pack_node(action, stack)
             # Бежим по пакам
             for pack in clazz.subpacks:
-                self._build_pack_node(pack, stack)
                 pack.controller = self
+                self._build_pack_node(pack, stack)
             stack.pop()
         else:
             clazz.controller = self
@@ -939,9 +960,13 @@ class ActionController(object):
             # Для быстрого поиска
             self._add_action_to_search_dicts(clazz, full_path)
 
-            self._url_patterns[full_path] = (stack[:], clazz)
+            self._url_patterns[full_path] = (stack[:], clazz, None)
+            for suff in getattr(clazz, 'suffixes', []):
+                self._url_patterns["%s:%s" % (full_path, suff)] = (
+                    stack[:], clazz, suff
+                )
 
-    def _invoke(self, request, action, stack):
+    def _invoke(self, request, action, stack, suffix=None):
         '''
         Непосредственный вызов экшена с отработкой всех событий
 
@@ -963,17 +988,15 @@ class ActionController(object):
 
             return OperationResult.by_message(msg)
 
-        # Заполняем контект и проверяем его
-        rules = action.context_declaration()
-        context = self.build_context(request, rules)
         try:
-            context.build(request, rules)
+            # Заполняем контект и проверяем его
+            context = action.build_context(request, suffix)
         except CriticalContextBuildingError:
             # критическая ошибка сбора контекста - должна валиться
             raise
         except ContextBuildingError as e:
             # некритичную ошибку - показываем пользователю
-            return OperationResult.by_message(text_type(e))
+            return OperationResult.by_message(unicode(e))
         except RequiredFailed as e:
             # если контекст неправильный, то возвращаем
             # фейльный результат операции
@@ -987,6 +1010,9 @@ class ActionController(object):
         request.target_packs = stack
         request.target_action = action
 
+        # контроллер может сам дополнять контекст
+        self.process_context(request, context, action)
+
         try:
             # Все ПРЕ обработчики
             for pack in stack:
@@ -997,7 +1023,10 @@ class ActionController(object):
             result = action.pre_run(request, context)
             if result is not None:
                 return result
-            response = action.run(request, context)
+            if suffix is None:
+                response = action.run(request, context)
+            else:
+                response = action.run(request, context, suffix)
             result = action.post_run(request, context, response)
             if result is not None:
                 return result
@@ -1033,15 +1062,14 @@ class ActionController(object):
         path = request.path
         matched = self._url_patterns.get(path)
         if matched:
-            stack, action = matched
+            stack, action, suffix = matched
 
             with _STATSD_CLIENT(self, request):
                 try:
-                    result = self._invoke(request, action, stack)
-                except:  # noqa
+                    result = self._invoke(request, action, stack, suffix)
+                except:
                     if settings.DEBUG:
-                        # Записывает сообщение в логгер если включен тестовый
-                        # режим
+                    # Записывает сообщение в логгер если включен тестовый режим
                         logger.exception(
                             u'ActionController.process_request: '
                             u'перехвачена необработанная ошибка'
@@ -1066,28 +1094,16 @@ class ActionController(object):
 
         raise http.Http404()
 
-    def build_context(self, request, rules):
-        '''
-        Выполняет построение контекста вызова операции ActionContext
-        на основе переданного request
+    def process_context(self, request, context, action):
+        """
+        Метод, позволяющий контроллерам дополнять контекст экшнов
+        перед их вызовом
+        """
+        pass
 
-        :param request: запрос
-        :type request: django.http.Request
-
-        :param dict rules: построение контекста
-
-        :return: пустой экземпляр контекста
-        :rtype: m3_core.actions.context.DeclarativeActionContext()
-            или m3_core.actions.context.ActionContext
-        '''
-        if DeclarativeActionContext.matches(rules):
-            return DeclarativeActionContext()
-        else:
-            return ActionContext()
-
-    # =========================================================================
+    #==========================================================================
     # Методы, предназначенные для поиска экшенов и паков в контроллере
-    # =========================================================================
+    #==========================================================================
     @_must_be_replaced_by('ControllerCache.find_pack')
     def find_pack(self, type):
         return self._find_pack(type)
@@ -1097,7 +1113,7 @@ class ActionController(object):
         return self._find_action(type)
 
     def _find_pack(self, pack):
-        if isinstance(pack, string_types):
+        if isinstance(pack, basestring):
             inst = self._packs_by_name.get(pack)
         elif inspect.isclass(pack):
             inst = self._packs_by_type.get(pack)
@@ -1106,7 +1122,7 @@ class ActionController(object):
         return inst
 
     def _find_action(self, action):
-        if isinstance(action, string_types):
+        if isinstance(action, basestring):
             inst, _ = self._actions_by_name.get(action, (None, None))
         elif inspect.isclass(action):
             inst, _ = self._actions_by_type.get(action, (None, None))
@@ -1135,10 +1151,10 @@ class ActionController(object):
         """
         return self._nodes_by_perm.get(perm)
 
-    # =========================================================================
+    #==========================================================================
     # Методы, предназначенные для добавления/изменения/удаления
     # пакетов действий в контроллер
-    # =========================================================================
+    #==========================================================================
     def append_pack(self, pack):
         """
         Добавляет пак в контроллер.
@@ -1151,13 +1167,13 @@ class ActionController(object):
                 u'Pack must be an instance of ActionPack subclass!')
 
         # пак может быть перегруже
-        if pack.get_short_name() in ControllerCache.overrides:
-            pack = ControllerCache.overrides[pack.get_short_name()]
+        if pack.short_name in ControllerCache.overrides:
+            pack = ControllerCache.overrides[pack.short_name]
 
-        self._build_pack_node(pack, [])
         if pack not in self.top_level_packs:
             self.top_level_packs.append(pack)
             pack.controller = self
+        self._build_pack_node(pack, [])
         ControllerCache.register_controller(self)
         return pack
 
@@ -1230,7 +1246,7 @@ class ActionController(object):
         left_packs = []
         right_packs = []
 
-        for url, value in iteritems(self._url_patterns):
+        for url, value in self._url_patterns.iteritems():
             packs_list, final_action = value
 
             # Поиск пака и соседей в списке
@@ -1319,7 +1335,7 @@ class ActionController(object):
         new_patterns = {}
         current_packs_slice = None
 
-        for url, value in iteritems(self._url_patterns):
+        for url, value in self._url_patterns.iteritems():
             packs_list, final_action = value
 
             # Поиск исходного пака и экшена в нём
@@ -1359,11 +1375,11 @@ class ActionController(object):
         Отладочный метод.
         Выводит в консоль список всех адрес зарегистрированных в контроллере.
         '''
-        print('==== CONTROLLER WITH URL: %s ======' % self.url)
+        print '==== CONTROLLER WITH URL: %s ======' % self.url
         for key in sorted(self._url_patterns.keys()):
-            print(key)
-        print()
-        print('Total patterns %s' % len(list(self._url_patterns.keys())))
+            print key
+        print
+        print 'Total patterns %s' % len(self._url_patterns.keys())
 
     def get_action_by_url(self, url):
         """
@@ -1381,7 +1397,7 @@ class ActionController(object):
         Получение списка действий или наборов, находящихся на первом уровне
         '''
         top_packs = []
-        for stack, act in self._url_patterns.values():
+        for stack, act, _ in self._url_patterns.values():
             if len(stack) == 0:
                 # значит экшин на верхнем уровне
                 if act not in top_packs:
@@ -1395,7 +1411,7 @@ class ActionController(object):
         '''
         Возвращение всех паков в контроллере
         '''
-        return list(self._packs_by_name.values())
+        return self._packs_by_name.values()
 
     def reset(self):
         '''
@@ -1441,15 +1457,15 @@ class ControllerCache(object):
 
     overrides = {}
 
-    # =========================================================================
+    #==========================================================================
     # Методы, предназначенные для поиска экшенов
     # и паков во всех контроллерах системы
-    # =========================================================================
+    #==========================================================================
 
     @classmethod
     def get_action_url(cls, type):
         """ Возвращает URL экшена *type* по его имени или классу """
-        assert isinstance(type, string_types) or issubclass(type, Action)
+        assert isinstance(type, basestring) or issubclass(type, Action)
         cls.populate()
         for cont in cls._controllers:
             url = cont.get_action_url(type)
@@ -1471,7 +1487,7 @@ class ControllerCache(object):
         """
         if inspect.isclass(pack):
             name = pack.get_short_name()
-        elif isinstance(pack, string_types) and '.' in pack:
+        elif isinstance(pack, str) and '.' in pack:
             name = pack
         else:
             raise ValueError(
@@ -1528,7 +1544,7 @@ class ControllerCache(object):
             if node:
                 return node
 
-    # =========================================================================
+    #==========================================================================
     @classmethod
     def _self_test(cls):
         """
@@ -1625,17 +1641,17 @@ class ControllerCache(object):
 
             cls.overrides = {}
             procs = []
-            for app_name in get_installed_apps():
+            for app_name in settings.INSTALLED_APPS:
                 try:
-                    module = importlib.import_module('.app_meta', app_name)
-                except ImportError as err:
+                    module = import_module('.app_meta', app_name)
+                except ImportError, err:
                     if err.args[0].find('No module named') == -1:
                         raise
                     continue
 
                 # расширение словаря overridde'ов
                 for original_cls, new_inst in (
-                    iteritems(getattr(module, 'action_pack_overrides', {}))
+                    getattr(module, 'action_pack_overrides', {}).iteritems()
                 ):
                     if inspect.isclass(original_cls):
                         original_cls = original_cls.get_short_name()
@@ -1661,7 +1677,7 @@ class ControllerCache(object):
         Отладочный метод.
         Выводит в консоль адреса всех контроллеров зарегистрированных в кэше.
         '''
-        print('------------ CONTROLLER CACHE DUMP ------------')
+        print '------------ CONTROLLER CACHE DUMP ------------'
         for cont in cls._controllers:
             cont.dump_urls()
 
