@@ -1,16 +1,22 @@
-# coding: utf-8
-u"""Хелперы для отработки расширяемых конфигураций url'ов."""
-from __future__ import absolute_import
+#coding: utf-8
+"""
+Хелперы для отработки расширяемых конфигураций url'ов
++++++++++++++++++++++++++++++++++++++++++++++++++++++
+"""
 
-from importlib import import_module
 import collections
 import inspect
+
 import warnings
 
+from django.conf import settings
+from django.utils import importlib
+from django.conf import urls
+
+
 from m3 import caching
-from m3.actions import Action
-from m3.actions import ControllerCache
-from m3_django_compat import get_installed_apps
+
+from m3.actions import ControllerCache, Action
 
 
 def _get_instance(obj):
@@ -28,19 +34,23 @@ def _get_instance(obj):
 
 
 def get_app_urlpatterns():
-    '''
+    """
     Возвращает конфигурацию урлов, объявленных в app_meta приложений.
 
     Данная функция не проглатывает ошибки, а выбрасывает все наружу.
     Перехват исключительных ситуаций данной фунции необходимо осуществлять
     вручную в urls.py прикладных приложений
-    '''
-    url_patterns = []
+    """
+    if getattr(urls, 'patterns', None):
+        # Поддержка django over 1.4
+        url_patterns = urls.patterns('',)
+    else:
+        url_patterns = urls.defaults.patterns('',)
 
-    for app_name in get_installed_apps():
+    for app_name in settings.INSTALLED_APPS:
         try:
-            module = import_module('.app_meta', app_name)
-        except ImportError as err:
+            module = importlib.import_module('.app_meta', app_name)
+        except ImportError, err:
             # по идее, такая ошибка возникает в
             # случае, если у нас для
             # установленного приложения
@@ -100,7 +110,6 @@ def get_url(action):
             break
     return action_data[1] if action_data else ''
 
-
 get_acton_url = get_url
 
 
@@ -132,9 +141,9 @@ def get_pack_by_url(url):
     return None
 
 
-# =============================================================================
+#==============================================================================
 # Кеш, используемый для хранения соответствия экшенов
-# =============================================================================
+#==============================================================================
 class ActionsNameCache(caching.IntegralRuntimeCache):
     '''
     Кеш, используемый для хранения соответствия имен экшенов и паков
